/*
 Stephan Kottler
 kottlers@informatik.uni-tuebingen.de
 *
 */

#ifndef IFSTREAM_HPP_
#define IFSTREAM_HPP_

#include <stdio.h>

class IFStream{
	/** The current character */
	char	cur;
	/** Pointer to the file to read from */
	FILE*   in;

	inline void skipWhitespace();

public:
	inline IFStream(char* file_name);
	inline ~IFStream();
	inline bool hasNext();
	inline void close();
	inline void skipLine();
	inline bool currEquals(const char& c);
	inline char getCurrent();
	inline bool currIsAlNum();
	inline bool isBad();
	inline bool operator==(const char* chrs);
	inline IFStream& operator++();
	inline int  getNextInt();

private:

	/** Prohibited: Copying of IFStream.*/
	IFStream(const IFStream& cp);
	IFStream& operator=(const IFStream& cp);

};


//Implementation

inline	  IFStream::~IFStream()					{ if(in != NULL) fclose(in);}

inline void IFStream::close()					{ if(in != NULL) fclose(in),in = NULL; }

inline bool IFStream::currEquals(const char& c)	{ return cur == c;}

inline char IFStream::getCurrent()				{ return cur;}

inline bool IFStream::currIsAlNum()				{ return isalnum(cur);}

inline bool IFStream::isBad()					{ return in == NULL;}

inline bool IFStream::hasNext()					{ skipWhitespace(); return cur != EOF;}


inline IFStream::IFStream(char* file_name): cur(0){
	in = fopen (file_name, "r");
	if(in != NULL) cur = fgetc(in);
}


inline void IFStream::skipWhitespace() {
	while (cur == 32 || (cur >= 9 && cur <= 13)){ cur = fgetc(in);}}


inline void IFStream::skipLine() {
	while(cur != EOF && cur != '\0'){
		if (cur == '\n') { cur = fgetc(in); return; }
		cur = fgetc(in);
	}
}


inline IFStream& IFStream::operator++(){cur = fgetc(in);return *this;}

inline bool IFStream::operator==(const char* chrs){
	while(*chrs != 0 && cur != EOF){
		if(cur != *chrs) return false;
		++chrs, cur = fgetc(in);
	}
	return cur != EOF;
}



inline int32_t IFStream::getNextInt(){
	int32_t sign	( 1 );
	skipWhitespace();

	if	  (cur == '-') sign = -1, cur = fgetc(in);
	else if (cur == '+') cur = fgetc(in);

	if (cur < '0' || cur > '9')
		printf("PARSE ERROR at character %c (ASCI: %u)\n", cur,(uint8_t)cur), exit(EXIT_FAILURE);

	int32_t val (cur - '0');
	cur = fgetc(in);
	while (cur >= '0' && cur <= '9'){
		val = val*10 + (cur - '0'), cur = fgetc(in);
	}
	return val * sign;
}


#define ifstream_hpp_version "IFStream.hpp:	   $Revision: 1.3 $"

#endif /*IFSTREAM_HPP_*/
